<?php

namespace Drupal\weather_api_plugin\Plugin\WeatherApiPlugin;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\weather_api_plugin\WeatherApiPluginBase;

/**
 * Weather Stack service.
 *
 * @WeatherApiPlugin(
 *   id="open_weather_map_plugin",
 *   label="OpenWeatherMap",
 *   base_url="https://api.openweathermap.org/data/2.5/weather",
 *   template="open_weather_map_theme",
 * )
 */
class OpenWeatherMap extends WeatherApiPluginBase {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'service_options' => [],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $config = $this->getConfiguration();
    $form['service_options']['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API key'),
      '#default_value' => isset($config['service_options']['api_key']) ? $config['service_options']['api_key'] : '',
      '#required' => TRUE,
      '#description' => $this->t('You can get API key on account page in the <a href="@weather-service-link">@weather-service service</a>.', [
        '@weather-service-link' => 'https://openweathermap.org/',
        '@weather-service' => $this->t('Open Weather Map'),
      ]),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->setConfiguration([
      'service_options' => [
        'api_key' => $form_state->getValue('api_key'),
      ],
    ]);
  }

  /**
   * {@inheritDoc}
   */
  public function build(string $city, string $uuid, bool $getFromCache = TRUE) {
    return [
      '#theme' => $this->getBuildTemplate(),
      '#weather_details' => $this->getWeatherDetails($city, $uuid, $getFromCache),
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function getWeatherDetails(string $city, string $uuid, bool $getFromCache) {
    $weather_api_requester = \Drupal::service('flexible_weather_api.weather_api_requester');
    $config = $this->getConfiguration();
    $language_id = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $api_key = isset($config['service_options']['api_key']) ? $config['service_options']['api_key'] : NULL;
    $query = [
      'q' => $city,
      'appid' => $api_key,
      'lang' => $language_id,
      'units' => 'metric',
    ];
    $service_options = [
      'api_key_required' => TRUE,
      'api_key' => $api_key,
      'query_params' => $query,
      'base_url' => $this->getBaseUrl(),
      'language' => $language_id,
      'uuid' => $uuid,
    ];

    $weather_details = $weather_api_requester->retrieveDataFromCache($this->getPluginId(), $service_options, $getFromCache);

    if (!isset($weather_details->error)) {
      return $weather_details;
    }

    return [];
  }

}
