<?php

namespace Drupal\flexible_weather_api\Plugin\WeatherApiLocationServicePlugin;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\flexible_weather_api\WeatherApiLocationServicePluginBase;
use GeoNames\Client as GeoNamesClient;

/**
 * The Geonames location service.
 *
 * @WeatherApiLocationServicePlugin(
 *   id="weather_api_location_service_geonames",
 *   label="Geonames",
 * )
 */
class WeatherApiLocationServiceGeonames extends WeatherApiLocationServicePluginBase {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'service_options' => [],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $config = $this->getConfiguration();
    $form['service_options']['username'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Username'),
      '#default_value' => isset($config['service_options']['username']) ? $config['service_options']['username'] : '',
      '#required' => TRUE,
      '#description' => $this->t('You should register on the <a href="@location-service-link">@location-service service</a> and using your login for this service.', [
        '@location-service-link' => 'https://www.geonames.org/',
        '@location-service' => $this->t('Geonames'),
      ]),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->setConfiguration([
      'service_options' => [
        'username' => $form_state->getValue('username'),
      ],
    ]);
  }

  /**
   * {@inheritDoc}
   */
  public function getAddress(string $text) {
    $config = $this->getConfiguration();
    $username = $config['service_options']['username'];
    $geonames_client = new GeoNamesClient($username);
    $language_id = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $addresses = $geonames_client->search([
      'q' => $text,
      'lang' => $language_id,
      'maxRows' => 10,
    ]);
    $matches = [];
    foreach ($addresses as $address) {
      $matches[] = [
        'value' => Html::escape($address->name),
        'label' => Html::escape($address->name),
      ];
    }

    return $matches;
  }

}
